(***********************************************************************)
(*                                                                     *)
(*                        Banana Algebra AST                           *)
(*                                                                     *)
(*                 Claus Brabrand, Jacob Andersen                      *)
(*               BRICS, University of Aarhus, Denmark                  *)
(*                                                                     *)
(*     (C)opyright 2007 BRICS / Basic Research in Computer Science     *)
(*                      All rights reserved                            *)
(*                                                                     *)
(***********************************************************************)


type 
pos = Lexing.position * Lexing.position

and
var = string

(* -- exceptions ----------------------------------------------------- *)

exception Error of string * pos
exception SourcedError of string * pos * pos
exception Conflict of string * pos * pos
exception SourcedConflict of string * pos * pos * pos

(* -- l -------------------------------------------------------------- *)

type
w = regexp * pos

and
term = var * regexp * pos

and
prod = var * var * entity list * pos

and
entity = Es of string * pos
       | En of var * pos

and
lcon = w * term list * prod list

(* -- regexp --------------------------------------------------------- *)

and
regexp = Union        of regexp * regexp * pos
       | Concat       of regexp * regexp * pos
       | Optional     of regexp * pos
       | Star         of regexp * pos
       | Plus         of regexp * pos
       | CharClass    of charclass list * pos
       | NegCharClass of charclass list * pos
       | Dot          of pos
       | Empty        of pos
       | String       of string * pos
       | RegexpID     of var * pos
       | Ch           of char * pos

and
charclass = CharExp of char * pos
          | Range   of char * char * pos


(* -- x -------------------------------------------------------------- *)

and
xprod = var * var * ast * pos

and
ast = Aast of var * var * ast list * pos
    | Aarg of int * pos
    | Aterm of var * string * pos
    | Aliteral of string * pos

and
tau = (string * string * pos) list

and
xcon = xprod list * tau * lcon * lcon


(* -- lexp ----------------------------------------------------------- *)

and
lexp = Lcon     of lcon * pos
     | Lvar     of var * pos
     | Llet     of var * lexp * lexp * pos
     | Lletx    of var * xexp * lexp * pos
     | Lplus    of lexp * lexp * pos
     | Lminus   of lexp * lexp * pos
     | Lsrc     of xexp * pos
     | Ltgt     of xexp * pos
     | Linclude of string * pos


(* -- xexp ----------------------------------------------------------- *)

and
xexp = Xcon     of xprod list * tau * lexp * lexp * pos
     | Xvar     of var * pos
     | Xlet     of var * lexp * xexp * pos
     | Xletx    of var * xexp * xexp * pos
     | Xplus    of xexp * xexp * pos
     | Xseq     of xexp * xexp * pos
     | Xidx     of lexp * pos
     | Xinclude of string * pos
