(***********************************************************************)
(*                                                                     *)
(*         Banana Algebra Code (generating: dk.brics.grammar)          *)
(*                                                                     *)
(*                         Claus Brabrand                              *)
(*               BRICS, University of Aarhus, Denmark                  *)
(*                                                                     *)
(*     (C)opyright 2005 BRICS / Basic Research in Computer Science     *)
(*                      All rights reserved                            *)
(*                                                                     *)
(***********************************************************************)


open Misc
open Ast
open List

(* -- code parse -------------------------------------------------- *)

let rec 
code_parse_entities n p es inx = match es with
    [] -> ()
  | Et(t,_)::es -> begin
                     println ("        _" ^ n ^ "_" ^ p ^ ".add(new Terminal(\"" ^ t ^ "\"));") ;
                     code_parse_entities n p es inx
                   end
  | En(m,_)::es -> begin
                     println ("        _" ^ n ^ "_" ^ p ^ ".add(new Nonterminal(\"" ^ m ^ "\", \"" ^ (int_to_string inx) ^ "\"));") ;
                     code_parse_entities n p es (inx+1)
                   end
  | Eo(o,_)::es -> begin
                     println ("        _" ^ n ^ "_" ^ p ^ ".add(new Nonterminal(\"" ^ o ^ "\"));") ;
                     code_parse_entities n p es inx
                   end
  | Eomit(_)::es -> begin
                     println ("        _" ^ n ^ "_" ^ p ^ ".add(new Terminal(_,false,\"_\"));") ;
                     code_parse_entities n p es inx
                   end

and
code_parse_prods ps = match ps with
    [] -> ()
  | (n,p,es,_)::ps -> 
      begin
        println ("        List<Entity> _" ^ n ^ "_" ^ p ^ " = new ArrayList<Entity>();") ;
        code_parse_entities n p es 0 ;
        println ("        productions.add(new Production(\"" ^ n ^ "\", _" ^ n ^ "_" ^ p ^ ", false, new ProductionID(\"" ^ p ^ "\")));") ;
        code_parse_prods ps
      end

and
code_parse start ps = 
  begin
    println "    protected static AST parse(String input) throws ParseException {" ;
    println "        Collection<Production> productions = new ArrayList<Production>();" ;
    code_parse_prods ps ;
    println ("        return (new Parser(new Grammar(\"" ^ start ^ "\", productions))).parse(input);") ;
    println "    }" ;
    println ""
  end

(* -- code transform -------------------------------------------------- *)

and
code_transform_ast a = match a with
    Aast(n,p,asts,_) -> 
      begin
        print ("new Ast(\"" ^ n ^ "\",\"" ^ p ^ "\",") ;
        code_transform_asts asts ;
        print ")"
      end
  | Aarg(k,inx,_) -> print ("transform(ast.asts.get(" ^ (int_to_string inx) ^ "))")

and
code_transform_asts asts = match asts with
    [] -> print "add()"
  | a::asts -> 
      begin
        print "add(" ;
        code_transform_ast a ;
        print "," ;
        code_transform_asts asts ;
        print ")"
      end

and
code_transform_xprods xps = match xps with
    [] -> ()
  | (n,p,a,_)::xps ->
      begin
        print ( "        if (ast.n.equals(\"" ^ n ^ "\") && ast.p.equals(\"" ^ p ^ "\")) return ") ;
        code_transform_ast a ;
        println ";" ;
        code_transform_xprods xps
      end

and
code_transform xps = 
  begin
    println "    protected static Ast transform(Ast ast) {" ;
    code_transform_xprods xps ;
    println "        throw new RuntimeException(\"!!!\");" ;
    println "    }" ;
    println ""
  end

(* -- code unparse -------------------------------------------------- *)

and
code_unparse_entities es inx = match es with
    [] -> ()
  | Et(t,_)::es -> begin
                     println ("            System.out.print(\"" ^ t ^ "\");") ;
                     code_unparse_entities es inx
                   end
  | En(m,_)::es -> begin
                     println ("            unparse(ast.asts.get(" ^ (int_to_string inx) ^ "));") ;
                     code_unparse_entities es (inx+1)
                   end
  | Eo(o,_)::es -> begin
                     code_unparse_entities es inx
                   end
  | Eomit(_)::es -> begin
                      println ("            System.out.print(\" \");") ;
                      code_unparse_entities es inx
                    end

and
code_unparse_prods ps = match ps with
    [] -> ()
  | (n,p,es,_)::ps ->
      begin
        println ( "        if (ast.n.equals(\"" ^ n ^ "\") && ast.p.equals(\"" ^ p ^ "\")) {") ;
        code_unparse_entities es 0 ;
        println "        }" ;
        code_unparse_prods ps
      end

and
code_unparse start ps = 
  begin
    println "    protected static void unparse(Ast ast) {" ;
    code_unparse_prods ps ;
    println "    }" ;
    println ""
  end

(* -- code main -------------------------------------------------- *)

and
code_header (start:string) = 
  begin
    println "import java.io.*;" ;
    println "import java.util.*;" ;
    println "import dk.brics.automaton.*;" ;
    println "import dk.brics.grammar.*;" ;
    println "import dk.brics.grammar.ast.*;" ;
    println "import dk.brics.grammar.parser.*;" ;
    println "" ;
    println "public class X {" ;
    println "    static Automaton _ = new RegExp(\"[\\n\\r\\t\\f ]*\").toAutomaton();" ;
    println "" 
  end

and
code_footer (start:string) = 
  begin
    println "    protected static List<Ast> add() {" ;
    println "        return new ArrayList<Ast>();" ;
    println "    }" ;
    println "" ;
    println "    protected static List<Ast> add(Ast ast, List<Ast> asts) {" ;
    println "        asts.add(0, ast);" ;
    println "        return asts;" ;
    println "    }" ;
    println "    protected static void printAst(AST ast) {" ;
    println "        printAst(ast.getRoot());" ;
    println "    }" ;
    println "" ;
    println "    protected static void printAst(Node ast) {" ;
    println "        String n = ast.getName();" ;
    println "        String p = ast.getLabel();" ;
    println "        System.out.print(n + \".\" + p + \"(\");" ;
    println "        for (String name : ast.getItemNames()) {" ;
    println "            Item item = ast.getItem(name);" ;
    println "            if (item instanceof Node) " ;
    println "  	             printAst((Node) item);" ;
    println "        }" ;
    println "        System.out.print(\")\");" ;
    println "    }" ;
    println "" ;
    println "    protected static Ast makeAst(AST ast) {" ;
    println "        return makeAst(ast.getRoot());" ;
    println "    }" ;
    println "" ;
    println "    protected static Ast makeAst(Node ast) {" ;
    println "        String n = ast.getName();" ;
    println "        String p = ast.getLabel();" ;
    println "        List<Ast> asts = new ArrayList<Ast>();" ;
    println "        for (String name : ast.getItemNames()) {" ;
    println "            Item item = ast.getItem(name);" ;
    println "            if (item instanceof Node) " ;
    println "  	             asts.add(makeAst((Node) item));" ;
    println "        }" ;
    println "        return new Ast(n,p,asts);" ;
    println "    }" ;
    println "" ;
    println "    public static String getString(String u) throws IOException {" ;
    println "        InputStream in = new FileInputStream(u);" ;
    println "        BufferedReader r = new BufferedReader(new InputStreamReader(in)); // assuming default encoding" ;
    println "        StringBuffer b = new StringBuffer();" ;
    println "        boolean done = false;" ;
    println "        while (!done) {" ;
    println "            int c = r.read();" ;
    println "            if (c == -1)" ;
    println "                done = true;" ;
    println "            else" ;
    println "                b.append((char)c);" ;
    println "        }" ;
    println "        in.close();" ;
    println "        return b.toString();" ;
    println "    }" ;
    println "" ;
    println "    public static void main(String[] args) {" ;
    println "        try {" ;
    println "            String input = getString(args[0]);" ;
    println "            unparse(transform(makeAst(parse(input))));" ;
    println "        } catch(IOException ex) {" ;
    println "            System.out.println(\"*** no such input file\");" ;
    println "        } catch(ParseException ex) {" ;
    println "            System.out.println(ex.getMessage());" ;
    println "        }" ;
    println "    }" ;
    println "}" 
  end

and
code (chi,tau,ls,lt) (start:string) = 
  begin
    code_header start ;
    code_parse start ls ;
    code_transform chi ;
    code_unparse (get_tau tau start) lt ;
    code_footer start
  end
