(***********************************************************************)
(*                                                                     *)
(*                 Banana Algebra InlineAuxLexer                       *)
(*                                                                     *)
(*                         Jacob Andersen                              *)
(*               BRICS, University of Aarhus, Denmark                  *)
(*                                                                     *)
(*     (C)opyright 2007 BRICS / Basic Research in Computer Science     *)
(*                      All rights reserved                            *)
(*                                                                     *)
(***********************************************************************)

{
  open InlineAuxParser
  open Lexing

let
  removecs s =
    if ((String.length s) mod 2) <> 0 then
      raise (Failure "Inline parser: lexing error 1")
    else
      begin
        let s' = String.make ((String.length s)/2) 'c' in
        let i = ref 0 in
        String.iter (fun c -> (if (!i mod 2) = 0 then
                                (if c <> 'c' then
                                   raise (Failure
                                          "Inline parser: lexing error 2"))
                               else
                                 s'.[(!i-1)/2] <- c) ;
                               i:=!i+1) s ;
        s'
      end

}

let space   = [' ' '\t' '\n' '\r']
let ignore  = "<?" ([^ '?'] | ('?' [^ '>']))* "?>"
let comment = "<!--" ([^ '-'] | ('-' [^ '-']) | ("--" [^ '>']))* "-->"
let tag     = '<' space* ('/' as close)? space*
              (((['a'-'z''A'-'Z''_']['a'-'z''A'-'Z''0'-'9''_']*)?) as n)
              ('.' (((['a'-'z''A'-'Z''_''.']['a'-'z''A'-'Z''0'-'9''_']*)?) as p))? 
              (space+ "value" space* '=' space* '"' 
                  (((['0'-'9']+) as num) | 
                   ((('c'_)+) as str) |
                   ('(' ((['a'-'z''A'-'Z''_']['a'-'z''A'-'Z''0'-'9''_']*) as n') ':' 
                        ((['0'-'9']+) as num) ')')) 
                '"')? space*
              ('/' as selfclose)? space* '>'


rule token = parse
  |  space | ignore | comment     
       {  
         token lexbuf
       }    
  |  tag
       {
         begin
           match n' with None -> () | Some s ->
             if (String.capitalize s) <> (String.capitalize n) then
               raise (Failure ("Inline parser: lexing error 3:'"^s^"','"^n^"'"))
         end ;
         
         match p, num, str with
             None, None, None ->
               if n="" then
                 raise (Failure "Inline parser: lexing error 4")
               else
                 begin
                  match close, selfclose with
                      None, None   -> N n
                    | Some _, None -> Nclose n
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 5")
                 end
           | Some p, None, None ->
               if n="" && p="" then 
                 begin
                  match close, selfclose with
                    | Some _, None -> Iclose
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 6")
                 end
               else if n="" || p="" then
                 raise (Failure "Inline parser: lexing error 7")
               else if p="." then
                 begin
                  match close, selfclose with
                      None, None   -> GAP n
                    | Some _, None -> GAPclose n
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 8")
                 end
               else
                 begin
                  match close, selfclose with
                      None, None   -> NP (n,p)
                    | Some _, None -> NPclose (n,p)
                    | None, Some _ -> NPselfclose (n,p)
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 9")
                 end
           | Some p, Some i, None ->
               if n="" && p="" then
                 begin
                  match close, selfclose with
                      None, None   -> I (int_of_string i)
                    | None, Some _ -> Iselfclose (int_of_string i)
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 10")
                 end
               else
                 raise (Failure "Inline parser: lexing error 11")
           | None, Some i, None ->
               if n="" then
                 raise (Failure "Inline parser: lexing error 12")
               else
                 begin
                  match close, selfclose with
                      None, None   -> NI (n, int_of_string i)
                    | None, Some _ -> NIselfclose (n, int_of_string i)
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 13")
                 end
           | None, None, Some s ->
               if n="" then
                 raise (Failure "Inline parser: lexing error 14")
               else
                 begin
                  match close, selfclose with
                      None, None   -> NS (n,removecs s)
                    | None, Some _ -> NSselfclose (n,removecs s)
                    | _            -> raise (Failure 
                                             "Inline parser: lexing error 15")
                 end
           | _ ->
               raise (Failure "Inline parser: lexing error 16")
       }
  |  _
       {
         raise (Failure "Inline parser: lexing error 17")
       }

{
 let token' buf = let t = token buf in begin (match t with  
   | EOF               -> print_endline "EOF"
   | Iclose            -> print_endline "IClose"
   | I i               -> print_endline ("I(" ^ (string_of_int i) ^")")
   | Iselfclose i      -> print_endline ("Iselfclose(" ^ (string_of_int i) ^")")
   | N s               -> print_endline ("N(" ^ s ^")")
   | Nclose s          -> print_endline ("Nclose(" ^ s ^")")
   | GAP s             -> print_endline ("GAP(" ^ s ^")")
   | GAPclose s        -> print_endline ("GAPclose(" ^ s ^")")
   | NP (n,p)          -> print_endline ("NP(" ^ n ^ "," ^ p ^ ")")
   | NPselfclose (n,p) -> print_endline ("NPselfclose(" ^ n ^ "," ^ p ^ ")")
   | NS (n,s)          -> print_endline ("NS(" ^ n ^ "," ^ s ^ ")")
   | NSselfclose (n,s) -> print_endline ("NSselfclose(" ^ n ^ "," ^ s ^ ")")
   | NPclose (n,p)     -> print_endline ("NPclose(" ^ n ^ "," ^ p ^ ")")
   | NI (n,i)          -> print_endline ("NI(" ^ n ^ "," ^ (string_of_int i) ^ ")")
   | NIselfclose (n,i) -> print_endline ("NIselfclose(" ^ n ^ "," ^ (string_of_int i) ^ ")")

  ) ; t end}
