/***********************************************************************/
/*                                                                     */
/*                 Banana Algebra InlineAuxParser                      */
/*                                                                     */
/*                         Jacob Andersen                              */
/*               BRICS, University of Aarhus, Denmark                  */
/*                                                                     */
/*     (C)opyright 2007 BRICS / Basic Research in Computer Science     */
/*                      All rights reserved                            */
/*                                                                     */
/***********************************************************************/


%{
  open Ast
  let dPos = (Lexing.dummy_pos, Lexing.dummy_pos)
%}


/* -- Lexeme declarations -------------------------------------------- */

%token                       EOF Iclose
%token <int>                 I Iselfclose
%token <string>              N Nclose GAP GAPclose
%token <string * string>     NP NPselfclose NS NSselfclose NPclose
%token <string * int>        NI NIselfclose

/* -- Entry ---------------------------------------------------------- */

%start ast
%type <Ast.ast> ast

%%


/* -- L constants ---------------------------------------------------- */

ast : N NPselfclose Nclose {let (n1,(n2,p),n3) = ($1,$2,$3) in
                            if n1=n2 && n2=n3 then Aast(n1,p,[],dPos) else raise(Parsing.Parse_error) }

    | N NP asts NPclose Nclose { let (n1,(n2,p1),(n3,p2),n4) = ($1,$2,$4,$5) in
                                 if n1=n2 && n2=n3 && n3=n4 && p1=p2 then
                                   Aast(n1,p1,$3,dPos)
                                 else
                                   raise(Parsing.Parse_error) }
    | NIselfclose              { Aarg (snd $1,dPos)}
    | NI Nclose                { let ((n1,i),n2) = ($1,$2) in
                                 if n1=n2 then
                                   Aarg(i,dPos) 
                                 else 
                                   raise(Parsing.Parse_error) }
    | N GAP Iselfclose GAPclose Nclose
                               { let (n1,n2,n3,n4) = ($1,$2,$4,$5) in
                                 if n1=n2 && n2=n3 && n3=n4 then
                                   Aarg($3,dPos)
                                 else
                                   raise(Parsing.Parse_error) }
    | N GAP I Iclose GAPclose Nclose
                               { let (n1,n2,n3,n4) = ($1,$2,$5,$6) in
                                 if n1=n2 && n2=n3 && n3=n4 then
                                   Aarg($3,dPos)
                                 else
                                   raise(Parsing.Parse_error) }
    | NSselfclose              { Aterm (fst $1,snd $1,dPos)}
    | NS Nclose                { let ((n1,s),n2) = ($1,$2) in
                                 if n1=n2 then
                                   Aterm(n1,s,dPos)
                                 else
                                   raise(Parsing.Parse_error) }

asts : ast asts                { $1 :: $2 }
     |                         { [] }

