(***********************************************************************)
(*                                                                     *)
(*                        Banana Algebra Main                          *)
(*                                                                     *)
(*                 Claus Brabrand, Jacob Andersen                      *)
(*               BRICS, University of Aarhus, Denmark                  *)
(*                                                                     *)
(*     (C)opyright 2007 BRICS / Basic Research in Computer Science     *)
(*                      All rights reserved                            *)
(*                                                                     *)
(***********************************************************************)

open Ast
open Parse
open Reduce
open Pretty
open CodeXML


(* -- Functions & Exceptions  ---------------------------------------- *)

let version = 
  "Banana Algebra v.0.99 (C)opyright 2007 BRICS, University of Aarhus, Denmark"

let print_usage () = 
  begin
    Printf.fprintf stderr "%s" (version ^ "\n\nUsage: banana <filename>\n") ;
    exit 0
  end

let string_of_pos (s,e) =
  s.Lexing.pos_fname ^ " (" ^ (string_of_int s.Lexing.pos_lnum) ^ "," ^
  (string_of_int ((s.Lexing.pos_cnum) - (s.Lexing.pos_bol))) ^ ")-(" ^
  (string_of_int e.Lexing.pos_lnum) ^ "," ^
  (string_of_int ((e.Lexing.pos_cnum) - (e.Lexing.pos_bol) - 1)) ^ ")"


(* -- Main  ---------------------------------------------------------- *)
 
let main =
  if (Array.length Sys.argv != 2) then 
    print_usage()
  else
    let 
      filename = Sys.argv.(1)
    in
      try
        let 
          xexp = parse filename
        in let 
          xcon = reduce xexp
        in
          begin
            (try
              let
                file = open_out ((Filename.chop_extension filename)
                                 ^ ".pretty")
              in
                begin
                  pretty_xcon xcon file ;
                  close_out file
                end
            with
              Sys_error msg -> raise (Failure 
                  ("Error writing transformation constant output file:\n"
                   ^ msg))) ;
            try
              code xcon filename
            with
              Sys_error msg ->
                  raise (Failure ("Error writing output file:\n" ^ msg))
          end 
      with Lexer.LexError (s,pos) ->
             prerr_endline ("*** In " ^ (string_of_pos pos) ^
                            ": Lexing error at token \"" ^ s ^ "\"")
         | ParseError(s,pos) ->
             prerr_endline ("*** In " ^ (string_of_pos pos) ^
                            ": Parse error at \"" ^ s ^ "\"")
         | Failure s -> prerr_endline ("*** FATAL: " ^ s)
         | FileNotFound (filename,pos) -> (match pos with
                None     -> prerr_endline ("*** File not found: " ^ filename) 
             |  Some pos -> prerr_endline ("*** In " ^ (string_of_pos pos) ^
                                           ": File not found: " ^ filename))
         | Error(s,pos) ->
             prerr_endline ("*** In " ^ (string_of_pos pos) ^ ": " ^ s)
         | SourcedError(s,p1,p2) ->
             prerr_endline ("*** In " ^ (string_of_pos p1) ^ " due to " ^
                            "problem at " ^ (string_of_pos p2) ^ ": " ^ s)
         | Conflict(s,p1,p2) ->
             prerr_endline ("*** Conflict between " ^ (string_of_pos p1) ^
                            " and " ^ (string_of_pos p2) ^ ": " ^ s) 
         | SourcedConflict(s,p1,p2,p3) ->
             prerr_endline ("*** In " ^ (string_of_pos p1) ^
                            ": Conflict between " ^ (string_of_pos p2) ^
                            " and " ^ (string_of_pos p3) ^ ": " ^ s) 
             

(*
    | SymbolError(msg, lineno) ->
        fprintln stderr ("*** symbol error\n    " ^ !filename ^ ":" ^ (int_to_string(lineno)) ^ ": " ^ msg)
    | TypeError(msg, lineno) ->
        fprintln stderr ("*** type error\n    " ^ !filename ^ ":" ^ (int_to_string(lineno)) ^ ": " ^ msg)
    | FatalError(msg) ->
        fprintln stderr ("*** fatal error\n    " ^ msg ^ "!!!")
*)
